@@ -9,6 +9,7 @@
 */
 
 #include "mz.h"
+#include "mz_crypt.h"
 
 #include <openssl/err.h>
 #include <openssl/engine.h>
@@ -19,14 +20,8 @@
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
 
-#if defined(MZ_ZIP_SIGNING)
-/* Note: https://www.imperialviolet.org/2015/10/17/boringssl.html says that
-   BoringSSL does not support CMS. "#include <etc/cms.h>" will fail. See
-   https://bugs.chromium.org/p/boringssl/issues/detail?id=421
-*/
-#include <openssl/cms.h>
-#include <openssl/pkcs12.h>
-#include <openssl/x509.h>
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#  include <openssl/core_names.h>
 #endif
 
 /***************************************************************************/
@@ -34,6 +29,7 @@
 static void mz_crypt_init(void) {
     static int32_t openssl_initialized = 0;
     if (!openssl_initialized) {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
         OpenSSL_add_all_algorithms();
 
         ERR_load_BIO_strings();
@@ -41,17 +37,16 @@ static void mz_crypt_init(void) {
 
         ENGINE_load_builtin_engines();
         ENGINE_register_all_complete();
+#else
+        OPENSSL_init_crypto(OPENSSL_INIT_ENGINE_ALL_BUILTIN, NULL);
+#endif
 
         openssl_initialized = 1;
     }
 }
 
 int32_t mz_crypt_rand(uint8_t *buf, int32_t size) {
-    int32_t result = 0;
-
-    result = RAND_bytes(buf, size);
-
-    if (!result)
+    if (!RAND_bytes(buf, size))
         return MZ_CRYPT_ERROR;
 
     return size;
@@ -60,32 +55,46 @@ int32_t mz_crypt_rand(uint8_t *buf, int32_t size) {
 /***************************************************************************/
 
 typedef struct mz_crypt_sha_s {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     union {
         SHA512_CTX ctx512;
         SHA256_CTX ctx256;
-        SHA_CTX    ctx1;
+        SHA_CTX ctx1;
     };
-    int32_t        initialized;
-    int32_t        error;
-    uint16_t       algorithm;
+#else
+    EVP_MD_CTX *ctx;
+#endif
+    unsigned long error;
+    int32_t initialized;
+    uint16_t algorithm;
 } mz_crypt_sha;
 
 /***************************************************************************/
 
 static const uint8_t mz_crypt_sha_digest_size[] = {
-    MZ_HASH_SHA1_SIZE,                     0, MZ_HASH_SHA224_SIZE,
-    MZ_HASH_SHA256_SIZE, MZ_HASH_SHA384_SIZE, MZ_HASH_SHA512_SIZE
-};
+    MZ_HASH_SHA1_SIZE, 0, MZ_HASH_SHA224_SIZE, MZ_HASH_SHA256_SIZE, MZ_HASH_SHA384_SIZE, MZ_HASH_SHA512_SIZE};
 
 /***************************************************************************/
 
+static void mz_crypt_sha_free(void *handle) {
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+    mz_crypt_sha *sha = (mz_crypt_sha *)handle;
+    if (sha->ctx)
+        EVP_MD_CTX_free(sha->ctx);
+    sha->ctx = NULL;
+#else
+    MZ_UNUSED(handle);
+#endif
+}
+
 void mz_crypt_sha_reset(void *handle) {
     mz_crypt_sha *sha = (mz_crypt_sha *)handle;
 
+    mz_crypt_init();
+    mz_crypt_sha_free(handle);
+
     sha->error = 0;
     sha->initialized = 0;
-
-    mz_crypt_init();
 }
 
 int32_t mz_crypt_sha_begin(void *handle) {
@@ -97,6 +106,7 @@ int32_t mz_crypt_sha_begin(void *handle) {
 
     mz_crypt_sha_reset(handle);
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     switch (sha->algorithm) {
     case MZ_HASH_SHA1:
         result = SHA1_Init(&sha->ctx1);
@@ -114,6 +124,33 @@ int32_t mz_crypt_sha_begin(void *handle) {
         result = SHA512_Init(&sha->ctx512);
         break;
     }
+#else
+    const EVP_MD *md = NULL;
+    switch (sha->algorithm) {
+    case MZ_HASH_SHA1:
+        md = EVP_sha1();
+        break;
+    case MZ_HASH_SHA224:
+        md = EVP_sha224();
+        break;
+    case MZ_HASH_SHA256:
+        md = EVP_sha256();
+        break;
+    case MZ_HASH_SHA384:
+        md = EVP_sha384();
+        break;
+    case MZ_HASH_SHA512:
+        md = EVP_sha512();
+        break;
+    }
+    if (!md)
+        return MZ_PARAM_ERROR;
+
+    sha->ctx = EVP_MD_CTX_new();
+    if (!sha->ctx)
+        return MZ_MEM_ERROR;
+    result = EVP_DigestInit_ex(sha->ctx, md, NULL);
+#endif
 
     if (!result) {
         sha->error = ERR_get_error();
@@ -131,6 +168,7 @@ int32_t mz_crypt_sha_update(void *handle, const void *buf, int32_t size) {
     if (!sha || !buf || !sha->initialized)
         return MZ_PARAM_ERROR;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     switch (sha->algorithm) {
     case MZ_HASH_SHA1:
         result = SHA1_Update(&sha->ctx1, buf, size);
@@ -148,6 +186,9 @@ int32_t mz_crypt_sha_update(void *handle, const void *buf, int32_t size) {
         result = SHA512_Update(&sha->ctx512, buf, size);
         break;
     }
+#else
+    result = EVP_DigestUpdate(sha->ctx, buf, size);
+#endif
 
     if (!result) {
         sha->error = ERR_get_error();
@@ -166,6 +207,7 @@ int32_t mz_crypt_sha_end(void *handle, uint8_t *digest, int32_t digest_size) {
     if (digest_size < mz_crypt_sha_digest_size[sha->algorithm - MZ_HASH_SHA1])
         return MZ_PARAM_ERROR;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     switch (sha->algorithm) {
     case MZ_HASH_SHA1:
         result = SHA1_Final(digest, &sha->ctx1);
@@ -183,6 +225,9 @@ int32_t mz_crypt_sha_end(void *handle, uint8_t *digest, int32_t digest_size) {
         result = SHA512_Final(digest, &sha->ctx512);
         break;
     }
+#else
+    result = EVP_DigestFinal_ex(sha->ctx, digest, NULL);
+#endif
 
     if (!result) {
         sha->error = ERR_get_error();
@@ -192,21 +237,18 @@ int32_t mz_crypt_sha_end(void *handle, uint8_t *digest, int32_t digest_size) {
     return MZ_OK;
 }
 
-void mz_crypt_sha_set_algorithm(void *handle, uint16_t algorithm) {
+int32_t mz_crypt_sha_set_algorithm(void *handle, uint16_t algorithm) {
     mz_crypt_sha *sha = (mz_crypt_sha *)handle;
-    if (MZ_HASH_SHA1 <= algorithm && algorithm <= MZ_HASH_SHA512)
-        sha->algorithm = algorithm;
+    if (algorithm < MZ_HASH_SHA1 || algorithm > MZ_HASH_SHA512)
+        return MZ_PARAM_ERROR;
+    sha->algorithm = algorithm;
+    return MZ_OK;
 }
 
-void *mz_crypt_sha_create(void **handle) {
-    mz_crypt_sha *sha = NULL;
-
-    sha = (mz_crypt_sha *)calloc(1, sizeof(mz_crypt_sha));
+void *mz_crypt_sha_create(void) {
+    mz_crypt_sha *sha = (mz_crypt_sha *)calloc(1, sizeof(mz_crypt_sha));
     if (sha)
         sha->algorithm = MZ_HASH_SHA256;
-    if (handle)
-        *handle = sha;
-
     return sha;
 }
 
@@ -216,7 +258,7 @@ void mz_crypt_sha_delete(void **handle) {
         return;
     sha = (mz_crypt_sha *)*handle;
     if (sha) {
-        mz_crypt_sha_reset(*handle);
+        mz_crypt_sha_free(*handle);
         free(sha);
     }
     *handle = NULL;
@@ -225,84 +267,197 @@ void mz_crypt_sha_delete(void **handle) {
 /***************************************************************************/
 
 typedef struct mz_crypt_aes_s {
-    AES_KEY    key;
-    int32_t    mode;
-    int32_t    error;
-    uint8_t    *key_copy;
-    int32_t    key_length;
+    int32_t mode;
+    unsigned long error;
+    EVP_CIPHER_CTX *ctx;
 } mz_crypt_aes;
 
 /***************************************************************************/
 
+static void mz_crypt_aes_free(void *handle) {
+    mz_crypt_aes *aes = (mz_crypt_aes *)handle;
+    if (aes->ctx)
+        EVP_CIPHER_CTX_free(aes->ctx);
+    aes->ctx = NULL;
+}
+
 void mz_crypt_aes_reset(void *handle) {
-    MZ_UNUSED(handle);
-
     mz_crypt_init();
+    mz_crypt_aes_free(handle);
 }
 
-int32_t mz_crypt_aes_encrypt(void *handle, uint8_t *buf, int32_t size) {
+int32_t mz_crypt_aes_encrypt(void *handle, const void *aad, int32_t aad_size, uint8_t *buf, int32_t size) {
     mz_crypt_aes *aes = (mz_crypt_aes *)handle;
 
-    if (!aes || !buf)
+    if (!aes || !buf || size % MZ_AES_BLOCK_SIZE != 0 || !aes->ctx)
         return MZ_PARAM_ERROR;
-    if (size != MZ_AES_BLOCK_SIZE)
+    if (aes->mode != MZ_AES_MODE_GCM && aad && aad_size > 0)
         return MZ_PARAM_ERROR;
 
-    AES_encrypt(buf, buf, &aes->key);
-    /* Equivalent to AES_ecb_encrypt with AES_ENCRYPT */
+    if (aad && aad_size > 0) {
+        int32_t how_many = 0;
+        if (!EVP_EncryptUpdate(aes->ctx, NULL, &how_many, aad, aad_size))
+            return MZ_CRYPT_ERROR;
+    }
+
+    if (!EVP_EncryptUpdate(aes->ctx, buf, &size, buf, size))
+        return MZ_CRYPT_ERROR;
+
     return size;
 }
 
-int32_t mz_crypt_aes_decrypt(void *handle, uint8_t *buf, int32_t size) {
+int32_t mz_crypt_aes_encrypt_final(void *handle, uint8_t *buf, int32_t size, uint8_t *tag, int32_t tag_size) {
     mz_crypt_aes *aes = (mz_crypt_aes *)handle;
-    if (!aes || !buf)
-        return MZ_PARAM_ERROR;
-    if (size != MZ_AES_BLOCK_SIZE)
+    int result = 0;
+    int out_len = 0;
+
+    if (!aes || !tag || !tag_size || !aes->ctx || aes->mode != MZ_AES_MODE_GCM)
         return MZ_PARAM_ERROR;
 
-    AES_decrypt(buf, buf, &aes->key);
-    /* Equivalent to AES_ecb_encrypt with AES_DECRYPT */
+    if (buf && size) {
+        if (!EVP_EncryptUpdate(aes->ctx, buf, &size, buf, size))
+            return MZ_CRYPT_ERROR;
+    }
+
+    /* Must call EncryptFinal for tag to be calculated */
+    result = EVP_EncryptFinal_ex(aes->ctx, NULL, &out_len);
+
+    if (result)
+        result = EVP_CIPHER_CTX_ctrl(aes->ctx, EVP_CTRL_GCM_GET_TAG, tag_size, tag);
+
+    if (!result) {
+        aes->error = ERR_get_error();
+        return MZ_CRYPT_ERROR;
+    }
+
     return size;
 }
 
-int32_t mz_crypt_aes_set_encrypt_key(void *handle, const void *key, int32_t key_length) {
+int32_t mz_crypt_aes_decrypt(void *handle, const void *aad, int32_t aad_size, uint8_t *buf, int32_t size) {
     mz_crypt_aes *aes = (mz_crypt_aes *)handle;
-    int32_t result = 0;
-    int32_t key_bits = 0;
 
-    if (!aes || !key || !key_length)
+    if (!aes || !buf || size % MZ_AES_BLOCK_SIZE != 0 || !aes->ctx)
+        return MZ_PARAM_ERROR;
+    if (aes->mode != MZ_AES_MODE_GCM && aad && aad_size > 0)
         return MZ_PARAM_ERROR;
 
-    mz_crypt_aes_reset(handle);
+    if (aad && aad_size > 0) {
+        int32_t how_many = 0;
+        if (!EVP_DecryptUpdate(aes->ctx, NULL, &how_many, aad, aad_size))
+            return MZ_CRYPT_ERROR;
+    }
 
-    key_bits = key_length * 8;
-    result = AES_set_encrypt_key(key, key_bits, &aes->key);
-    if (result) {
+    if (!EVP_DecryptUpdate(aes->ctx, buf, &size, buf, size))
+        return MZ_CRYPT_ERROR;
+
+    return size;
+}
+
+int32_t mz_crypt_aes_decrypt_final(void *handle, uint8_t *buf, int32_t size, const uint8_t *tag, int32_t tag_length) {
+    mz_crypt_aes *aes = (mz_crypt_aes *)handle;
+    int out_len = 0;
+
+    if (!aes || !tag || !tag_length || !aes->ctx || aes->mode != MZ_AES_MODE_GCM)
+        return MZ_PARAM_ERROR;
+
+    if (buf && size) {
+        if (!EVP_DecryptUpdate(aes->ctx, buf, &size, buf, size))
+            return MZ_CRYPT_ERROR;
+    }
+
+    /* Set expected tag */
+    if (!EVP_CIPHER_CTX_ctrl(aes->ctx, EVP_CTRL_GCM_SET_TAG, tag_length, (void *)tag)) {
+        aes->error = ERR_get_error();
+        return MZ_CRYPT_ERROR;
+    }
+
+    /* Must call DecryptFinal for tag verification */
+    if (!EVP_DecryptFinal_ex(aes->ctx, NULL, &out_len)) {
+        aes->error = ERR_get_error();
+        return MZ_CRYPT_ERROR;
+    }
+
+    return size;
+}
+
+static int32_t mz_crypt_aes_set_key(void *handle, const void *key, int32_t key_length, const void *iv,
+                                    int32_t iv_length, int32_t encrypt) {
+    mz_crypt_aes *aes = (mz_crypt_aes *)handle;
+    const EVP_CIPHER *type = NULL;
+
+    switch (aes->mode) {
+    case MZ_AES_MODE_CBC:
+        if (key_length == 16)
+            type = EVP_aes_128_cbc();
+        else if (key_length == 24)
+            type = EVP_aes_192_cbc();
+        else if (key_length == 32)
+            type = EVP_aes_256_cbc();
+        break;
+    case MZ_AES_MODE_ECB:
+        if (key_length == 16)
+            type = EVP_aes_128_ecb();
+        else if (key_length == 24)
+            type = EVP_aes_192_ecb();
+        else if (key_length == 32)
+            type = EVP_aes_256_ecb();
+        break;
+    case MZ_AES_MODE_GCM:
+        if (key_length == 16)
+            type = EVP_aes_128_gcm();
+        else if (key_length == 24)
+            type = EVP_aes_192_gcm();
+        else if (key_length == 32)
+            type = EVP_aes_256_gcm();
+        break;
+    }
+    if (!type)
+        return MZ_PARAM_ERROR;
+
+    aes->ctx = EVP_CIPHER_CTX_new();
+    if (!aes->ctx)
+        return MZ_MEM_ERROR;
+
+    if (!EVP_CipherInit_ex(aes->ctx, type, NULL, key, iv, encrypt)) {
         aes->error = ERR_get_error();
         return MZ_HASH_ERROR;
     }
 
+    EVP_CIPHER_CTX_set_padding(aes->ctx, aes->mode == MZ_AES_MODE_GCM);
+
     return MZ_OK;
 }
 
-int32_t mz_crypt_aes_set_decrypt_key(void *handle, const void *key, int32_t key_length) {
+int32_t mz_crypt_aes_set_encrypt_key(void *handle, const void *key, int32_t key_length, const void *iv,
+                                     int32_t iv_length) {
     mz_crypt_aes *aes = (mz_crypt_aes *)handle;
-    int32_t result = 0;
-    int32_t key_bits = 0;
 
     if (!aes || !key || !key_length)
         return MZ_PARAM_ERROR;
+    if (key_length != 16 && key_length != 24 && key_length != 32)
+        return MZ_PARAM_ERROR;
+    if (iv && iv_length != MZ_AES_BLOCK_SIZE)
+        return MZ_PARAM_ERROR;
 
     mz_crypt_aes_reset(handle);
 
-    key_bits = key_length * 8;
-    result = AES_set_decrypt_key(key, key_bits, &aes->key);
-    if (result) {
-        aes->error = ERR_get_error();
-        return MZ_HASH_ERROR;
-    }
+    return mz_crypt_aes_set_key(handle, key, key_length, iv, iv_length, 1);
+}
 
-    return MZ_OK;
+int32_t mz_crypt_aes_set_decrypt_key(void *handle, const void *key, int32_t key_length, const void *iv,
+                                     int32_t iv_length) {
+    mz_crypt_aes *aes = (mz_crypt_aes *)handle;
+
+    if (!aes || !key || !key_length)
+        return MZ_PARAM_ERROR;
+    if (key_length != 16 && key_length != 24 && key_length != 32)
+        return MZ_PARAM_ERROR;
+    if (iv && iv_length > MZ_AES_BLOCK_SIZE)
+        return MZ_PARAM_ERROR;
+
+    mz_crypt_aes_reset(handle);
+
+    return mz_crypt_aes_set_key(handle, key, key_length, iv, iv_length, 0);
 }
 
 void mz_crypt_aes_set_mode(void *handle, int32_t mode) {
@@ -310,13 +465,8 @@ void mz_crypt_aes_set_mode(void *handle, int32_t mode) {
     aes->mode = mode;
 }
 
-void *mz_crypt_aes_create(void **handle) {
-    mz_crypt_aes *aes = NULL;
-
-    aes = (mz_crypt_aes *)calloc(1, sizeof(mz_crypt_aes));
-    if (handle)
-        *handle = aes;
-
+void *mz_crypt_aes_create(void) {
+    mz_crypt_aes *aes = (mz_crypt_aes *)calloc(1, sizeof(mz_crypt_aes));
     return aes;
 }
 
@@ -325,23 +475,31 @@ void mz_crypt_aes_delete(void **handle) {
     if (!handle)
         return;
     aes = (mz_crypt_aes *)*handle;
-    if (aes)
+    if (aes) {
+        mz_crypt_aes_free(*handle);
         free(aes);
+    }
     *handle = NULL;
 }
 
 /***************************************************************************/
 
 typedef struct mz_crypt_hmac_s {
-    HMAC_CTX   *ctx;
-    int32_t    initialized;
-    int32_t    error;
-    uint16_t   algorithm;
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    HMAC_CTX *ctx;
+#else
+    EVP_MAC *mac;
+    EVP_MAC_CTX *ctx;
+#endif
+    unsigned long error;
+    int32_t initialized;
+    uint16_t algorithm;
 } mz_crypt_hmac;
 
 /***************************************************************************/
 
-#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || (defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER < 0x2070000fL))
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) ||                                                                          \
+    (defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER < 0x2070000fL))
 static HMAC_CTX *HMAC_CTX_new(void) {
     HMAC_CTX *ctx = OPENSSL_malloc(sizeof(HMAC_CTX));
     if (ctx)
@@ -359,35 +517,74 @@ static void HMAC_CTX_free(HMAC_CTX *ctx) {
 
 /***************************************************************************/
 
+static void mz_crypt_hmac_free(void *handle) {
+    mz_crypt_hmac *hmac = (mz_crypt_hmac *)handle;
+
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    HMAC_CTX_free(hmac->ctx);
+#else
+    if (hmac->ctx)
+        EVP_MAC_CTX_free(hmac->ctx);
+    if (hmac->mac)
+        EVP_MAC_free(hmac->mac);
+    hmac->mac = NULL;
+#endif
+
+    hmac->ctx = NULL;
+}
+
 void mz_crypt_hmac_reset(void *handle) {
     mz_crypt_hmac *hmac = (mz_crypt_hmac *)handle;
 
-    HMAC_CTX_free(hmac->ctx);
-
-    hmac->ctx = NULL;
-    hmac->error = 0;
-
     mz_crypt_init();
+    mz_crypt_hmac_free(handle);
+
+    hmac->error = 0;
 }
 
 int32_t mz_crypt_hmac_init(void *handle, const void *key, int32_t key_length) {
     mz_crypt_hmac *hmac = (mz_crypt_hmac *)handle;
     int32_t result = 0;
-    const EVP_MD *evp_md = NULL;
 
     if (!hmac || !key)
         return MZ_PARAM_ERROR;
 
     mz_crypt_hmac_reset(handle);
 
-    hmac->ctx = HMAC_CTX_new();
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    const EVP_MD *evp_md = NULL;
 
     if (hmac->algorithm == MZ_HASH_SHA1)
         evp_md = EVP_sha1();
     else
         evp_md = EVP_sha256();
 
+    hmac->ctx = HMAC_CTX_new();
+    if (!hmac->ctx)
+        return MZ_MEM_ERROR;
+
     result = HMAC_Init_ex(hmac->ctx, key, key_length, evp_md, NULL);
+#else
+    char *digest_algorithm = NULL;
+    OSSL_PARAM params[2];
+
+    if (hmac->algorithm == MZ_HASH_SHA1)
+        digest_algorithm = "sha1";
+    else
+        digest_algorithm = "sha256";
+
+    params[0] = OSSL_PARAM_construct_utf8_string(OSSL_MAC_PARAM_DIGEST, digest_algorithm, 0);
+    params[1] = OSSL_PARAM_construct_end();
+
+    hmac->mac = EVP_MAC_fetch(NULL, "HMAC", NULL);
+    if (!hmac->mac)
+        return MZ_MEM_ERROR;
+    hmac->ctx = EVP_MAC_CTX_new(hmac->mac);
+    if (!hmac->ctx)
+        return MZ_MEM_ERROR;
+    result = EVP_MAC_init(hmac->ctx, key, key_length, params);
+#endif
+
     if (!result) {
         hmac->error = ERR_get_error();
         return MZ_HASH_ERROR;
@@ -403,7 +600,11 @@ int32_t mz_crypt_hmac_update(void *handle, const void *buf, int32_t size) {
     if (!hmac || !buf)
         return MZ_PARAM_ERROR;
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     result = HMAC_Update(hmac->ctx, buf, size);
+#else
+    result = EVP_MAC_update(hmac->ctx, buf, size);
+#endif
     if (!result) {
         hmac->error = ERR_get_error();
         return MZ_HASH_ERROR;
@@ -419,6 +620,7 @@ int32_t mz_crypt_hmac_end(void *handle, uint8_t *digest, int32_t digest_size) {
     if (!hmac || !digest)
         return MZ_PARAM_ERROR;
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     if (hmac->algorithm == MZ_HASH_SHA1) {
         if (digest_size < MZ_HASH_SHA1_SIZE)
             return MZ_BUF_ERROR;
@@ -429,6 +631,12 @@ int32_t mz_crypt_hmac_end(void *handle, uint8_t *digest, int32_t digest_size) {
             return MZ_BUF_ERROR;
         result = HMAC_Final(hmac->ctx, digest, (uint32_t *)&digest_size);
     }
+#else
+    {
+        size_t digest_outsize = digest_size;
+        result = EVP_MAC_final(hmac->ctx, digest, &digest_outsize, digest_size);
+    }
+#endif
 
     if (!result) {
         hmac->error = ERR_get_error();
@@ -446,34 +654,34 @@ void mz_crypt_hmac_set_algorithm(void *handle, uint16_t algorithm) {
 int32_t mz_crypt_hmac_copy(void *src_handle, void *target_handle) {
     mz_crypt_hmac *source = (mz_crypt_hmac *)src_handle;
     mz_crypt_hmac *target = (mz_crypt_hmac *)target_handle;
-    int32_t result = 0;
 
     if (!source || !target)
         return MZ_PARAM_ERROR;
 
     mz_crypt_hmac_reset(target_handle);
 
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     if (!target->ctx)
         target->ctx = HMAC_CTX_new();
 
-    result = HMAC_CTX_copy(target->ctx, source->ctx);
-    if (!result) {
+    if (!HMAC_CTX_copy(target->ctx, source->ctx)) {
         target->error = ERR_get_error();
         return MZ_HASH_ERROR;
     }
+#else
+    if (!target->ctx)
+        target->ctx = EVP_MAC_CTX_dup(source->ctx);
+    if (!target->ctx)
+        return MZ_MEM_ERROR;
+#endif
 
     return MZ_OK;
 }
 
-void *mz_crypt_hmac_create(void **handle) {
-    mz_crypt_hmac *hmac = NULL;
-
-    hmac = (mz_crypt_hmac *)calloc(1, sizeof(mz_crypt_hmac));
+void *mz_crypt_hmac_create(void) {
+    mz_crypt_hmac *hmac = (mz_crypt_hmac *)calloc(1, sizeof(mz_crypt_hmac));
     if (hmac)
         hmac->algorithm = MZ_HASH_SHA256;
-    if (handle)
-        *handle = hmac;
-
     return hmac;
 }
 
@@ -483,191 +691,8 @@ void mz_crypt_hmac_delete(void **handle) {
         return;
     hmac = (mz_crypt_hmac *)*handle;
     if (hmac) {
-        mz_crypt_hmac_reset(*handle);
+        mz_crypt_hmac_free(*handle);
         free(hmac);
     }
     *handle = NULL;
 }
-
-/***************************************************************************/
-
-#if defined(MZ_ZIP_SIGNING)
-int32_t mz_crypt_sign(uint8_t *message, int32_t message_size, uint8_t *cert_data, int32_t cert_data_size,
-    const char *cert_pwd, uint8_t **signature, int32_t *signature_size) {
-    PKCS12 *p12 = NULL;
-    EVP_PKEY *evp_pkey = NULL;
-    BUF_MEM *buf_mem = NULL;
-    BIO *cert_bio = NULL;
-    BIO *message_bio = NULL;
-    BIO *signature_bio = NULL;
-    CMS_ContentInfo *cms = NULL;
-    CMS_SignerInfo *signer_info = NULL;
-    STACK_OF(X509) *ca_stack = NULL;
-    X509 *cert = NULL;
-    int32_t result = 0;
-    int32_t err = MZ_OK;
-
-    if (!message || !cert_data || !signature || !signature_size)
-        return MZ_PARAM_ERROR;
-
-    mz_crypt_init();
-
-    *signature = NULL;
-    *signature_size = 0;
-
-    cert_bio = BIO_new_mem_buf(cert_data, cert_data_size);
-
-    if (!d2i_PKCS12_bio(cert_bio, &p12))
-        err = MZ_SIGN_ERROR;
-    if (err == MZ_OK)
-        result = PKCS12_parse(p12, cert_pwd, &evp_pkey, &cert, &ca_stack);
-    if (result) {
-        cms = CMS_sign(NULL, NULL, ca_stack, NULL, CMS_BINARY | CMS_PARTIAL);
-        if (cms)
-            signer_info = CMS_add1_signer(cms, cert, evp_pkey, EVP_sha256(), 0);
-        if (!signer_info) {
-            err = MZ_SIGN_ERROR;
-        } else {
-            message_bio = BIO_new_mem_buf(message, message_size);
-            signature_bio = BIO_new(BIO_s_mem());
-
-            result = CMS_final(cms, message_bio, NULL, CMS_BINARY);
-            if (result)
-                result = i2d_CMS_bio(signature_bio, cms);
-            if (result) {
-                BIO_flush(signature_bio);
-                BIO_get_mem_ptr(signature_bio, &buf_mem);
-
-                *signature_size = buf_mem->length;
-                *signature = malloc(buf_mem->length);
-
-                memcpy(*signature, buf_mem->data, buf_mem->length);
-            }
-#if 0
-            BIO *yy = BIO_new_file("xyz", "wb");
-            BIO_write(yy, *signature, *signature_size);
-            BIO_flush(yy);
-            BIO_free(yy);
-#endif
-        }
-    }
-
-    if (!result)
-        err = MZ_SIGN_ERROR;
-
-    if (cms)
-        CMS_ContentInfo_free(cms);
-    if (signature_bio)
-        BIO_free(signature_bio);
-    if (cert_bio)
-        BIO_free(cert_bio);
-    if (message_bio)
-        BIO_free(message_bio);
-    if (p12)
-        PKCS12_free(p12);
-
-    if (err != MZ_OK && *signature) {
-        free(*signature);
-        *signature = NULL;
-        *signature_size = 0;
-    }
-
-    return err;
-}
-
-int32_t mz_crypt_sign_verify(uint8_t *message, int32_t message_size, uint8_t *signature, int32_t signature_size) {
-    CMS_ContentInfo *cms = NULL;
-    STACK_OF(X509) *signers = NULL;
-    STACK_OF(X509) *intercerts = NULL;
-    X509_STORE *cert_store = NULL;
-    X509_LOOKUP *lookup = NULL;
-    X509_STORE_CTX *store_ctx = NULL;
-    BIO *message_bio = NULL;
-    BIO *signature_bio = NULL;
-    BUF_MEM *buf_mem = NULL;
-    int32_t signer_count = 0;
-    int32_t result = 0;
-    int32_t i = 0;
-    int32_t err = MZ_SIGN_ERROR;
-
-    if (!message || !message_size || !signature || !signature_size)
-        return MZ_PARAM_ERROR;
-
-    mz_crypt_init();
-
-    cert_store = X509_STORE_new();
-
-    X509_STORE_load_locations(cert_store, "cacert.pem", NULL);
-    X509_STORE_set_default_paths(cert_store);
-
-#if 0
-    BIO *yy = BIO_new_file("xyz", "wb");
-    BIO_write(yy, signature, signature_size);
-    BIO_flush(yy);
-    BIO_free(yy);
-#endif
-
-    lookup = X509_STORE_add_lookup(cert_store, X509_LOOKUP_file());
-    if (lookup)
-        X509_LOOKUP_load_file(lookup, "cacert.pem", X509_FILETYPE_PEM);
-    lookup = X509_STORE_add_lookup(cert_store, X509_LOOKUP_hash_dir());
-    if (lookup)
-        X509_LOOKUP_add_dir(lookup, NULL, X509_FILETYPE_DEFAULT);
-
-    signature_bio = BIO_new_mem_buf(signature, signature_size);
-    message_bio = BIO_new(BIO_s_mem());
-
-    cms = d2i_CMS_bio(signature_bio, NULL);
-    if (cms) {
-        result = CMS_verify(cms, NULL, cert_store, NULL, message_bio, CMS_NO_SIGNER_CERT_VERIFY | CMS_BINARY);
-        if (result)
-            signers = CMS_get0_signers(cms);
-        if (signers)
-            intercerts = CMS_get1_certs(cms);
-        if (intercerts) {
-            /* Verify signer certificates */
-            signer_count = sk_X509_num(signers);
-            if (signer_count > 0)
-                err = MZ_OK;
-
-            for (i = 0; i < signer_count; i++) {
-                store_ctx = X509_STORE_CTX_new();
-                X509_STORE_CTX_init(store_ctx, cert_store, sk_X509_value(signers, i), intercerts);
-                result = X509_verify_cert(store_ctx);
-                if (store_ctx)
-                    X509_STORE_CTX_free(store_ctx);
-
-                if (!result) {
-                    err = MZ_SIGN_ERROR;
-                    break;
-                }
-            }
-        }
-
-        BIO_get_mem_ptr(message_bio, &buf_mem);
-
-        if (err == MZ_OK) {
-            /* Verify the message */
-            if (((int32_t)buf_mem->length != message_size) ||
-                (memcmp(buf_mem->data, message, message_size) != 0))
-                err = MZ_SIGN_ERROR;
-        }
-    }
-
-#if 0
-    if (!result)
-        printf(ERR_error_string(ERR_get_error(), NULL));
-#endif
-
-    if (cms)
-        CMS_ContentInfo_free(cms);
-    if (message_bio)
-        BIO_free(message_bio);
-    if (signature_bio)
-        BIO_free(signature_bio);
-    if (cert_store)
-        X509_STORE_free(cert_store);
-
-    return err;
-}
-#endif
