@@ -21,13 +21,17 @@
 #  define SYMBOLIC_LINK_FLAG_DIRECTORY 0x1
 #endif
 
+#ifndef _WIN32_WINNT_WIN8
+#  define _WIN32_WINNT_WIN8 0x0602
+#endif
+
 /***************************************************************************/
 
 typedef struct DIR_int_s {
-    void            *find_handle;
+    void *find_handle;
     WIN32_FIND_DATAW find_data;
-    struct dirent    entry;
-    uint8_t          end;
+    struct dirent entry;
+    uint8_t end;
 } DIR_int;
 
 /***************************************************************************/
@@ -54,18 +58,18 @@ void mz_os_unicode_string_delete(wchar_t **string) {
     }
 }
 
-uint8_t *mz_os_utf8_string_create(const char *string, int32_t encoding) {
+char *mz_os_utf8_string_create(const char *string, int32_t encoding) {
     wchar_t *string_wide = NULL;
-    uint8_t *string_utf8 = NULL;
+    char *string_utf8 = NULL;
     uint32_t string_utf8_size = 0;
 
     string_wide = mz_os_unicode_string_create(string, encoding);
     if (string_wide) {
         string_utf8_size = WideCharToMultiByte(CP_UTF8, 0, string_wide, -1, NULL, 0, NULL, NULL);
-        string_utf8 = (uint8_t *)calloc(string_utf8_size + 1, sizeof(wchar_t));
+        string_utf8 = (char *)calloc(string_utf8_size + 1, sizeof(char));
 
         if (string_utf8)
-            WideCharToMultiByte(CP_UTF8, 0, string_wide, -1, (char *)string_utf8, string_utf8_size, NULL, NULL);
+            WideCharToMultiByte(CP_UTF8, 0, string_wide, -1, string_utf8, string_utf8_size, NULL, NULL);
 
         mz_os_unicode_string_delete(&string_wide);
     }
@@ -73,22 +77,22 @@ uint8_t *mz_os_utf8_string_create(const char *string, int32_t encoding) {
     return string_utf8;
 }
 
-uint8_t *mz_os_utf8_string_create_from_unicode(const wchar_t *string, int32_t encoding) {
-    uint8_t *string_utf8 = NULL;
+char *mz_os_utf8_string_create_from_unicode(const wchar_t *string, int32_t encoding) {
+    char *string_utf8 = NULL;
     uint32_t string_utf8_size = 0;
 
     MZ_UNUSED(encoding);
 
     string_utf8_size = WideCharToMultiByte(CP_UTF8, 0, string, -1, NULL, 0, NULL, NULL);
-    string_utf8 = (uint8_t *)calloc(string_utf8_size + 1, sizeof(wchar_t));
+    string_utf8 = (char *)calloc(string_utf8_size + 1, sizeof(char));
 
     if (string_utf8)
-        WideCharToMultiByte(CP_UTF8, 0, string, -1, (char *)string_utf8, string_utf8_size, NULL, NULL);
+        WideCharToMultiByte(CP_UTF8, 0, string, -1, string_utf8, string_utf8_size, NULL, NULL);
 
     return string_utf8;
 }
 
-void mz_os_utf8_string_delete(uint8_t **string) {
+void mz_os_utf8_string_delete(char **string) {
     if (string) {
         free(*string);
         *string = NULL;
@@ -230,8 +234,7 @@ static void mz_os_unix_to_file_time(time_t unix_time, FILETIME *file_time) {
 }
 
 int32_t mz_os_get_file_date(const char *path, time_t *modified_date, time_t *accessed_date, time_t *creation_date) {
-    WIN32_FIND_DATAW ff32;
-    HANDLE handle = NULL;
+    WIN32_FILE_ATTRIBUTE_DATA wfad;
     wchar_t *path_wide = NULL;
     int32_t err = MZ_INTERNAL_ERROR;
 
@@ -241,21 +244,19 @@ int32_t mz_os_get_file_date(const char *path, time_t *modified_date, time_t *acc
     if (!path_wide)
         return MZ_PARAM_ERROR;
 
-    handle = FindFirstFileW(path_wide, &ff32);
-    free(path_wide);
-
-    if (handle != INVALID_HANDLE_VALUE) {
+    if (GetFileAttributesExW(path_wide, GetFileExInfoStandard, &wfad)) {
         if (modified_date)
-            mz_os_file_to_unix_time(ff32.ftLastWriteTime, modified_date);
+            mz_os_file_to_unix_time(wfad.ftLastWriteTime, modified_date);
         if (accessed_date)
-            mz_os_file_to_unix_time(ff32.ftLastAccessTime, accessed_date);
+            mz_os_file_to_unix_time(wfad.ftLastAccessTime, accessed_date);
         if (creation_date)
-            mz_os_file_to_unix_time(ff32.ftCreationTime, creation_date);
+            mz_os_file_to_unix_time(wfad.ftCreationTime, creation_date);
 
-        FindClose(handle);
         err = MZ_OK;
     }
 
+    free(path_wide);
+
     return err;
 }
 
@@ -397,7 +398,7 @@ DIR *mz_os_open_dir(const char *path) {
     return (DIR *)dir_int;
 }
 
-struct dirent* mz_os_read_dir(DIR *dir) {
+struct dirent *mz_os_read_dir(DIR *dir) {
     DIR_int *dir_int;
 
     if (!dir)
@@ -407,8 +408,8 @@ struct dirent* mz_os_read_dir(DIR *dir) {
     if (dir_int->end)
         return NULL;
 
-    WideCharToMultiByte(CP_UTF8, 0, dir_int->find_data.cFileName, -1,
-        dir_int->entry.d_name, sizeof(dir_int->entry.d_name), NULL, NULL);
+    WideCharToMultiByte(CP_UTF8, 0, dir_int->find_data.cFileName, -1, dir_int->entry.d_name,
+                        sizeof(dir_int->entry.d_name), NULL, NULL);
 
     if (FindNextFileW(dir_int->find_handle, &dir_int->find_data) == 0) {
         if (GetLastError() != ERROR_NO_MORE_FILES)
@@ -433,6 +434,10 @@ int32_t mz_os_close_dir(DIR *dir) {
     return MZ_OK;
 }
 
+int32_t mz_os_is_dir_separator(const char c) {
+    return c == '\\' || c == '/';
+}
+
 int32_t mz_os_is_dir(const char *path) {
     wchar_t *path_wide = NULL;
     uint32_t attribs = 0;
@@ -476,7 +481,7 @@ int32_t mz_os_is_symlink(const char *path) {
 }
 
 int32_t mz_os_make_symlink(const char *path, const char *target_path) {
-    typedef BOOLEAN (WINAPI *LPCREATESYMBOLICLINKW)(LPCWSTR, LPCWSTR, DWORD);
+    typedef BOOLEAN(WINAPI * LPCREATESYMBOLICLINKW)(LPCWSTR, LPCWSTR, DWORD);
     MEMORY_BASIC_INFORMATION mbi;
     LPCREATESYMBOLICLINKW create_symbolic_link_w = NULL;
     HMODULE kernel32_mod = NULL;
@@ -488,7 +493,7 @@ int32_t mz_os_make_symlink(const char *path, const char *target_path) {
     if (!path)
         return MZ_PARAM_ERROR;
 
-    // Use VirtualQuery instead of GetModuleHandleW for UWP
+    /* Use VirtualQuery instead of GetModuleHandleW for UWP */
     memset(&mbi, 0, sizeof(mbi));
     VirtualQuery(VirtualQuery, &mbi, sizeof(mbi));
     kernel32_mod = (HMODULE)mbi.AllocationBase;
@@ -526,7 +531,7 @@ int32_t mz_os_make_symlink(const char *path, const char *target_path) {
 
 int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_target_path) {
     typedef struct _REPARSE_DATA_BUFFER {
-        ULONG  ReparseTag;
+        ULONG ReparseTag;
         USHORT ReparseDataLength;
         USHORT Reserved;
         union {
@@ -535,18 +540,18 @@ int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_targ
                 USHORT SubstituteNameLength;
                 USHORT PrintNameOffset;
                 USHORT PrintNameLength;
-                ULONG  Flags;
-                WCHAR  PathBuffer[1];
+                ULONG Flags;
+                WCHAR PathBuffer[1];
             } SymbolicLinkReparseBuffer;
             struct {
                 USHORT SubstituteNameOffset;
                 USHORT SubstituteNameLength;
                 USHORT PrintNameOffset;
                 USHORT PrintNameLength;
-                WCHAR  PathBuffer[1];
+                WCHAR PathBuffer[1];
             } MountPointReparseBuffer;
             struct {
-                UCHAR  DataBuffer[1];
+                UCHAR DataBuffer[1];
             } GenericReparseBuffer;
         };
     } REPARSE_DATA_BUFFER;
@@ -559,7 +564,7 @@ int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_targ
     int32_t target_path_len = 0;
     int32_t target_path_idx = 0;
     int32_t err = MZ_OK;
-    uint8_t *target_path_utf8 = NULL;
+    char *target_path_utf8 = NULL;
 
     if (!path)
         return MZ_PARAM_ERROR;
@@ -579,7 +584,7 @@ int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_targ
     handle = CreateFile2(path_wide, FILE_READ_EA, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_EXISTING, &extended_params);
 #else
     handle = CreateFileW(path_wide, FILE_READ_EA, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-        FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, NULL);
+                         FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, NULL);
 #endif
 
     if (handle == INVALID_HANDLE_VALUE) {
@@ -589,8 +594,7 @@ int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_targ
 
     if (DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0, buffer, sizeof(buffer), &length, NULL) == TRUE) {
         reparse_data = (REPARSE_DATA_BUFFER *)buffer;
-        if ((IsReparseTagMicrosoft(reparse_data->ReparseTag)) &&
-            (reparse_data->ReparseTag == IO_REPARSE_TAG_SYMLINK)) {
+        if ((IsReparseTagMicrosoft(reparse_data->ReparseTag)) && (reparse_data->ReparseTag == IO_REPARSE_TAG_SYMLINK)) {
             target_path_len = max_target_path * sizeof(wchar_t);
             if (target_path_len > reparse_data->SymbolicLinkReparseBuffer.PrintNameLength)
                 target_path_len = reparse_data->SymbolicLinkReparseBuffer.PrintNameLength;
@@ -599,16 +603,16 @@ int32_t mz_os_read_symlink(const char *path, char *target_path, int32_t max_targ
             if (target_path_wide) {
                 target_path_idx = reparse_data->SymbolicLinkReparseBuffer.PrintNameOffset / sizeof(wchar_t);
                 memcpy(target_path_wide, &reparse_data->SymbolicLinkReparseBuffer.PathBuffer[target_path_idx],
-                    target_path_len);
+                       target_path_len);
 
                 target_path_wide[target_path_len / sizeof(wchar_t)] = 0;
                 target_path_utf8 = mz_os_utf8_string_create_from_unicode(target_path_wide, MZ_ENCODING_UTF8);
 
                 if (target_path_utf8) {
-                    strncpy(target_path, (const char *)target_path_utf8, max_target_path - 1);
+                    strncpy(target_path, target_path_utf8, max_target_path - 1);
                     target_path[max_target_path - 1] = 0;
                     /* Ensure directories have slash at the end so we can recreate them later */
-                    if (mz_os_is_dir((const char *)target_path_utf8) == MZ_OK)
+                    if (mz_os_is_dir(target_path_utf8) == MZ_OK)
                         mz_path_append_slash(target_path, max_target_path, MZ_PATH_SLASH_PLATFORM);
                     mz_os_utf8_string_delete(&target_path_utf8);
                 } else {
